=begin

Permission to use, copy, modify, and distribute this software for
any purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies.

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

Name:         uploadWithThumbnail.rb
Author:       Andreas Eisenbarth
Usage:        menu Plugins > Upload model with custom thumbnail
              menu Plugins > Keep the current 3D Warehouse thumbnail

Description:  This plugin allows to create a thumbnail image for models
              shared on 3D Warehouse. It gives two options,
              1. to upload a custom image chosen from the file dialog,
              2. to keep the exact current image of the model in 3D Warehouse
                (requires wget)

              This is probably only of use for people who
              cannot export 2d images due to technical issues.

Date:         12.01.2013
Version:      2.1
=end
require 'sketchup.rb'



module AE



module UploadThumb



# Get a temporary folder that is writable
temp = [ENV['TMPDIR'], ENV['TMP'], ENV['TEMP'], ENV['USERPROFILE'], '/tmp', '.'].inject(nil){ |t,dir| (!t && dir && File.directory?(dir) && File.writable?(dir))? dir : t }
@temp = File.expand_path(temp).unpack("U*").map{|c|c.chr}.join
@wine_prefix = ENV["WINEPREFIX"] || "/home/#{ENV['USER']}/.wine"



# Keeps the current model thumbnail from 3D Warehouse. Since SketchUp will always
# upload and overwrite the thumbnail, we download the original first, and replace
# the SketchUp-generated one.
#
def self.keep
  model = Sketchup.active_model
  path = File.join(@temp, "thumbnail.jpg")
  mid = model.get_attribute("3DWarehouse","ModelID")
  return self.set if !mid
  url = "http://sketchup.google.com/3dwarehouse/download?mid=#{mid}&rtyp=lt"
  cmd = "wget \"#{url}\" --output-document=\"#{@wine_prefix}/dosdevices/#{path.sub(/^(\w)(?=\:)/,@temp[0,1].downcase)}\" &"
  sh = File.join(@temp, "temporary.sh")
  File.open(sh, "w"){|f|
    f.puts %[ #!/bin/sh]
    f.puts cmd
    f.puts %[exit 0]
  }
  begin
    #%x[Z:\\bin\\bash "#{@wine_prefix}/dosdevices/#{sh.sub(/^(\w)(?=\:)/,@temp[0,1].downcase)}"]
    system(%[Z:\\bin\\bash "#{@wine_prefix}/dosdevices/#{sh.sub(/^(\w)(?=\:)/,@temp[0,1].downcase)}"])
  rescue
  end
  self.observer(path){
    self.replace(path, true)
  }
end



# Allows the user to select a jpeg file and set it as thumbnail for the current 3D Warheouse upload.
#
def self.set
  path = UI.openpanel("Open thumbnail image", "c:\\", "*.jpg")
  return if path.nil?
  path = File.expand_path( path.unpack("U*").map{|c|c.chr}.join )
  return unless File.exists?(path)
  self.replace(path)
end



# Replaces the current thumbnail at the location where SketchUp stores temporary files for 3D Warehouse uploads.
#   @param [String] path  a jpeg image path
#   @param [Boolean] del  if a file path is given, whether to delete the file
#
def self.replace(path, del=false)
  sel = Sketchup.active_model.selection
  temp_snapshot = Dir.entries(@temp)
  # Check if there is a selection, then upload component or group.
  if sel.length==1 && ( sel[0].is_a?(Sketchup::ComponentInstance) || sel[0].is_a?(Sketchup::Group) )
    (x=sel[0].to_component; sel.add(x)) if sel[0].is_a?(Sketchup::Group)
    Sketchup.send_action("uploadComponent:") # 24188
  # Upload the complete model.
  else
    Sketchup.send_action("shareModel:") # 21989
  end
  # Wait until SketchUp exports files to the temporary folder for upload.
  criterium = Proc.new{ # a newly created file/folder with /skp/
    new_files = Dir.entries(@temp) - temp_snapshot
    !new_files.find_all{|f| f[/^skp/]}.empty?
  }
  self.observer(criterium, 200){
    new_files = Dir.entries(@temp) - temp_snapshot
    newest = new_files.inject( ".." ){|result, element| File.mtime(File.join(@temp,element)) > File.mtime(File.join(@temp,result)) ? element : result }
    # Move the custom thumbnail into the upload folder.
    self.syscopy( path, File.join(@temp, newest, "thumbnail.jpg") )
    File.delete(path) if del==true && File.exists?(path)
  }
end # def replace



# Observe when a criterium becomes true (especially a file is created).
#   @param [String, Proc] criterium  a file path string or a Proc returning true/false
#   @param [Numeric] counter  an optional security feature, stop observer if unsuccessful after many repetitions
#   @param [Boolean] del  if a file path is given, whether to delete the file
#   @param [Proc] callback  code block to execute when file is created
#
def self.observer(criterium, counter=200, del=false, &callback)
  timer = UI.start_timer(0.2, true){
    if counter > 0 && ( criterium.class == String && !FileTest.exist?(criterium.unpack("U*").map{|c|c.chr}.join) || criterium.class == Proc && criterium.call==false )
      counter -= 1
    elsif counter <= 0
      UI.stop_timer(timer)
      puts(Module.nesting[0].to_s+": Timeout error")
    else # if criterium.call == true
      UI.stop_timer(timer)
      File.delete(criterium) if del == true && criterium.class == String
      callback.call
    end
  }
end # def observer



# For copying files



# From ftools.rb, line 44
def self.catname(from, to)
  if File.directory?(to)
    File.join( to.sub(%r([/\\]$), ''), File.basename(from) )
  else
    to
  end
end #def



# From ftools.rb, line 56
def self.syscopy(from, to)
  to = self.catname(from, to)
  fmode = File.stat(from).mode
  tpath = to
  not_exist = !File.exist?(tpath)
  s = File.size(from)#
  from = File.open(from, "rb")
  to = File.open(to, "wb")
  begin
    while true
      # to.syswrite from.sysread(BUFSIZE)
      to.syswrite from.sysread(s)
    end
  rescue EOFError
    ret = true
  rescue
    ret = false
  ensure
    to.close
    from.close
  end
  File.chmod(fmode, tpath) if not_exist
  ret
end # def



end # module



end # module



if( not file_loaded?(File.basename(__FILE__)) )
  uwt_menu = UI.menu("File").add_submenu("Upload model with custom thumbnail")

  # Menu item to keep the current thumbnail (under the condition that wget is available).
  if File.exists?("Z:/usr/bin/wget")
    keep = uwt_menu.add_item("Keep the current 3D Warehouse thumbnail"){ AE::UploadThumb.keep }
    # Enable this menu entry only for models that are already on 3D Warehouse.
    uwt_menu.set_validation_proc(keep){
       if !Sketchup.active_model.get_attribute("3DWarehouse","ModelID").nil?
         MF_ENABLED
       else
         MF_GRAYED
       end
     }
  end

  # Menu item to set a custom thumbnail.
  uwt_menu.add_item("Choose an image file"){ AE::UploadThumb.set }

  file_loaded(File.basename(__FILE__))
end
